;------------------------------ DDS3.asm ------------------------------------

;Software 3-channel DDS implementation module
;using AT90PWM3B PWM DACs.

;DDS sample frequency is 8 MHz * 8 (PLL) / 2^12 = 15.625 kHz
;DDS frequency resolution is 15.625 kHz / 2^25 = 0.00046566 Hz

;----------------------------- Constantes: ----------------------------------

.equ	TAB_SIZE = 258	        ;nfble size, bytes

;----------------------------- Variables: -----------------------------------

.DSEG	;data segment (internal SRAM)

FreeMem:

.org	(FreeMem + 0xFF) & 0xFF00 ;align table to page

LUT:	.byte TAB_SIZE		;LUT

AFreq:  .byte 3                 ;A frequency code
BFreq:  .byte 3                 ;B frequency code
CFreq:  .byte 3                 ;C frequency code

;------------------------------ Macros: -------------------------------------

;Stop DDS:

.macro	StopDDS
	sts	PIM0,zero	;disable PSC0 End Cycle interrupt
.endm

;Start DDS:

.macro	StartDDS
	ldi	temp,(1<<PEOP0)	
	sts	PIFR0,temp	;clear pending interrupt
	ldi	temp,(1<<PEOPE0)	
	sts	PIM0,temp	;enable PSC0 End Cycle interrupt
.endm

;-------------------------------- Code: -------------------------------------

.CSEG	;Code segment

;PSC0 End Cycle interrupt (DDS implementation).
;To minimize execution time handle must be located
;direct at vector address. It eliminates RJMP command.

;xFreq - current frequency (memory variables)
;xPhaseL,M,N,P - current phase (register variables)
;SinL,H - sin samples magnitude (register variables)
;ZL,ZH - data pointer
;tsreg - SREG store
;r0,r1 - used with mul instruction

                                ;4      address store
DDS:    in	tsreg,SREG	;1      save status register

;Channel A:

        lds     r0,AFreq+0      ;2      Phase(0..24) = Phase(0..24) + Freq(0..22)
	add	APhaseL,r0	;1
        lds     r0,AFreq+1      ;2
	adc	APhaseM,r0      ;1
        lds     r0,AFreq+2      ;2
	adc	APhaseN,r0      ;1
	adc	APhaseP,zero    ;1

	mov	ZL,APhaseN	;1      ZL = wa (word address)
	mov	r0,APhaseM	;1      r0 = dx
	sbrc	APhaseN,7	;2/1    if(Phase.23 == 0)
	com	ZL		;0/1    wa = !wa
	sbrc	APhaseN,7	;2/1    if(Phase.23 == 0)
	com	r0		;0/1    dx = !dx

	ldi	ZH,high(LUT)	;1      ZH = table base (low(LUT) = 0)
        lsl     ZL              ;1      Z - low byte address

	ld	SinL,Z+		;2      SinL = lo sin[x]
	ld	SinH,Z+		;2      SinH = hi sin[x]
	ld	r1,Z		;2      r1 = lo sin[x + 1]
	sub	r1,SinL		;1      r1 = dA
	mul	r1,r0		;2      r1,r0 = dA * dx
	rol	r0		;1      C = 1 if r0.7 == 1
	adc	SinL,r1		;1      SinH:SinL = sin[x] + round(r1:r0 / 256)
	adc	SinH,zero	;1      SinH:SinL = A

	sbrs	APhaseP,0       ;2/1	
	rjmp	pa_ab1		;0/2    jump if Phase.24 == 0

pa_cd1:	com	SinL		;1/0    SIN < 0,
	com	SinH		;1/0    SinH:SinL = !SinH:SinL
	rjmp	pa_ad1          ;2/0
	
pa_ab1:	sec                     ;0/1
	sbc	SinL,zero       ;0/1    SIN > 0,
	sbc	SinH,zero	;0/1    SinH:SinL = SinH:SinL - 1

pa_ad1: subi    SinL,0xff       ;1      convert to unsigned
        sbci    SinH,0x07       ;1
        sts     OCR1SBH,SinH    ;2   
        sts     OCR1SBL,SinL    ;2 = 42 
        
;Channel B:

        lds     r0,BFreq+0      ;2      Phase(0..24) = Phase(0..24) + Freq(0..22)
	add	BPhaseL,r0	;1
        lds     r0,BFreq+1      ;2
	adc	BPhaseM,r0      ;1
        lds     r0,BFreq+2      ;2
	adc	BPhaseN,r0      ;1
	adc	BPhaseP,zero    ;1

	mov	ZL,BPhaseN	;1      ZL = wa (word address)
	mov	r0,BPhaseM	;1      r0 = dx
	sbrc	BPhaseN,7	;2/1    if(Phase.23 == 0)
	com	ZL		;0/1    wa = !wa
	sbrc	BPhaseN,7	;2/1    if(Phase.23 == 0)
	com	r0		;0/1    dx = !dx

	ldi	ZH,high(LUT)	;1      ZH = table base (low(LUT) = 0)
        lsl     ZL              ;1      Z - low byte address

	ld	SinL,Z+		;2      SinL = lo sin[x]
	ld	SinH,Z+		;2      SinH = hi sin[x]
	ld	r1,Z		;2      r1 = lo sin[x + 1]
	sub	r1,SinL		;1      r1 = dA
	mul	r1,r0		;2      r1,r0 = dA * dx
	rol	r0		;1      C = 1 if r0.7 == 1
	adc	SinL,r1		;1      SinH:SinL = sin[x] + round(r1:r0 / 256)
	adc	SinH,zero	;1      SinH:SinL = A

	sbrs	BPhaseP,0       ;2/1	
	rjmp	pb_ab1		;0/2    jump if Phase.24 == 0

pb_cd1:	com	SinL		;1/0    SIN < 0,
	com	SinH		;1/0    SinH:SinL = !SinH:SinL
	rjmp	pb_ad1          ;2/0
	
pb_ab1:	sec                     ;0/1
	sbc	SinL,zero       ;0/1    SIN > 0,
	sbc	SinH,zero	;0/1    SinH:SinL = SinH:SinL - 1

pb_ad1: subi    SinL,0xff       ;1      convert to unsigned
        sbci    SinH,0x07       ;1
        sts     OCR2SAH,SinH    ;2
        sts     OCR2SAL,SinL    ;2 = 42

;Channel C:

        lds     r0,CFreq+0      ;2      Phase(0..24) = Phase(0..24) + Freq(0..22)
	add	CPhaseL,r0	;1
        lds     r0,CFreq+1      ;2
	adc	CPhaseM,r0      ;1
        lds     r0,CFreq+2      ;2
	adc	CPhaseN,r0      ;1
	adc	CPhaseP,zero    ;1

	mov	ZL,CPhaseN	;1      ZL = wa (word address)
	mov	r0,CPhaseM	;1      r0 = dx
	sbrc	CPhaseN,7	;2/1    if(Phase.23 == 0)
	com	ZL		;0/1    wa = !wa
	sbrc	CPhaseN,7	;2/1    if(Phase.23 == 0)
	com	r0		;0/1    dx = !dx

	ldi	ZH,high(LUT)	;1      ZH = table base (low(LUT) = 0)
        lsl     ZL              ;1      Z - low byte address

	ld	SinL,Z+		;2      SinL = lo sin[x]
	ld	SinH,Z+		;2      SinH = hi sin[x]
	ld	r1,Z		;2      r1 = lo sin[x + 1]
	sub	r1,SinL		;1      r1 = dA
	mul	r1,r0		;2      r1,r0 = dA * dx
	rol	r0		;1      C = 1 if r0.7 == 1
	adc	SinL,r1		;1      SinH:SinL = sin[x] + round(r1:r0 / 256)
	adc	SinH,zero	;1      SinH:SinL = A

	sbrs	CPhaseP,0       ;2/1	
	rjmp	pc_ab1		;0/2    jump if Phase.24 == 0

pc_cd1: com	SinL		;1/0    SIN < 0,
	com	SinH		;1/0    SinH:SinL = !SinH:SinL
	rjmp	pc_ad1          ;2/0
	
pc_ab1: sec                     ;0/1
	sbc	SinL,zero       ;0/1    SIN > 0,
	sbc	SinH,zero	;0/1    SinH:SinL = SinH:SinL - 1

pc_ad1: subi    SinL,0xff       ;1      convert to unsigned
        sbci    SinH,0x07       ;1
        sts     OCR0SAH,SinH    ;2
        sts     OCR0SAL,SinL    ;2 = 42

        out	SREG,tsreg	;1      restore status register
	reti                    ;4

                                ;136 total, 17 uS @ 8 MHz
	
;----------------------------------------------------------------------------

;Init DDS subsystem:

iDDS:

;PLL enable:

        ldi     temp,(1<<PLLF) | (1<<PLLE)
        out     PLLCSR,temp

;Delay 100 ms:

        ldi     cnt,100
de2:

;Delay 1 ms:
	
        ldy	F_CLK/10000
de1:	sbiw	YH:YL,1         ;2
	nop                     ;1
	nop                     ;1
	nop                     ;1
	nop                     ;1
	nop                     ;1
	nop                     ;1
        brne    de1             ;2

        dec     cnt
        brne    de2

;Build sin LUT in RAM:

	table	SinTab
	ldy	LUT
SCopy:	lpm	temp,Z+
	st	Y+,temp
	cpi	YH,high(LUT + TAB_SIZE)
	brne	SCopy
	cpi	YL, low(LUT + TAB_SIZE)
	brne	SCopy

;Variables init

	clr	SinL		;clear sin amplitude
	clr	SinH

	clr	APhaseL         ;clear phase for channel A
	clr	APhaseM
	clr	APhaseN
	clr	APhaseP

	clr	BPhaseL         ;clear phase for channel B
	clr	BPhaseM
	clr	BPhaseN
	clr	BPhaseP

	clr	CPhaseL         ;clear phase for channel C
	clr	CPhaseM
	clr	CPhaseN
	clr	CPhaseP

        sts     AFreq+0,zero    ;clear frequency for channel A     
        sts     AFreq+1,zero     
        sts     AFreq+2,zero     

        sts     BFreq+0,zero    ;clear frequency for channel B     
        sts     BFreq+1,zero     
        sts     BFreq+2,zero     

        sts     CFreq+0,zero    ;clear frequency for channel C     
        sts     CFreq+1,zero     
        sts     CFreq+2,zero     

#ifdef TEST ;(50 Hz @ 8 MHz = 0x1A36E):

	ldi	temp,0x00       ;Fa = 50 Hz
        sts     AFreq+0,temp     
	ldi	temp,0xA7
        sts     AFreq+1,temp     
	ldi	temp,0x01
        sts     AFreq+2,temp     

	ldi	temp,0x00       ;Fb = 50 Hz
        sts     BFreq+0,temp     
	ldi	temp,0xA7
        sts     BFreq+1,temp     
	ldi	temp,0x01
        sts     BFreq+2,temp     

	ldi	temp,0x00       ;Fc = 50 Hz
        sts     CFreq+0,temp     
	ldi	temp,0xA7
        sts     CFreq+1,temp     
	ldi	temp,0x01
        sts     CFreq+2,temp     

	ldi	temp,0x00       ;Phase A = 0
	mov	APhaseL,temp
	ldi	temp,0x00
	mov	APhaseM,temp
	ldi	temp,0x00
	mov	APhaseN,temp
	ldi	temp,0x00
	mov	APhaseP,temp

	ldi	temp,0x55       ;Phase B = 120
	mov	BPhaseL,temp
	ldi	temp,0x55
	mov	BPhaseM,temp
	ldi	temp,0x55
	mov	BPhaseN,temp
	ldi	temp,0x01
	mov	BPhaseP,temp

	ldi	temp,0xAA       ;Phase C = 240
	mov	CPhaseL,temp
	ldi	temp,0xAA
	mov	CPhaseM,temp
	ldi	temp,0xAA
	mov	CPhaseN,temp
	ldi	temp,0x02
	mov	CPhaseP,temp

#endif

;Timer 1 setup:
        
	ldy     0x0fff

        sts     OCR0RAH,YH
        sts     OCR0RAL,YL
        sts     OCR0RBH,YH
        sts     OCR0RBL,YL

        sts     OCR1RAH,YH
        sts     OCR1RAL,YL
        sts     OCR1RBH,YH
        sts     OCR1RBL,YL

        sts     OCR2RAH,YH
        sts     OCR2RAL,YL
        sts     OCR2RBH,YH
        sts     OCR2RBL,YL

        ldy     0x0000

        sts     OCR0SBH,YH      ;clear amplitude A
        sts     OCR0SBL,YL
        sts     OCR2SBH,YH      ;clear amplitude B
        sts     OCR2SBL,YL
        sts     OCR1SAH,YH      ;clear amplitude C
        sts     OCR1SAL,YL

#ifdef TEST ;(Max. amplitude = 0xFFF)

        ldy     0x0fff

        sts     OCR0SBH,YH      ;amplitude A
        sts     OCR0SBL,YL
        sts     OCR2SBH,YH      ;amplitude B
        sts     OCR2SBL,YL
        sts     OCR1SAH,YH      ;amplitude C
        sts     OCR1SAL,YL

#endif

        ;PSC0 outputs enable:
        ldi     temp,(1<<POEN0B) | (1<<POEN0A)
        sts     PSOC0,temp
        ldi     temp,(1<<POEN1B) | (1<<POEN1A)
        sts     PSOC1,temp
        ldi     temp,(1<<POEN2B) | (1<<POEN2A)
        sts     PSOC2,temp

        ;PSC0 auto lock, 1 ramp, PLL clock
#ifdef ALIGN ;Center Aligned Mode PWM
        ldi     temp,(1<<PCLKSEL0) | (1<<PMODE00) | (1<<PMODE01)
        sts     PCNF0,temp
        ldi     temp,(1<<PCLKSEL1) | (1<<PMODE10) | (1<<PMODE11) | (1<<POP1)
        sts     PCNF1,temp
        ldi     temp,(1<<PCLKSEL2) | (1<<PMODE20) | (1<<PMODE21)
        sts     PCNF2,temp
#else        ;Fast PWM
        ldi     temp,(1<<PCLKSEL0)
        sts     PCNF0,temp
        ldi     temp,(1<<PCLKSEL1)
        sts     PCNF1,temp
        ldi     temp,(1<<PCLKSEL2)
        sts     PCNF2,temp
#endif

        ;CK/1, PSC1,PSC2 - slave, PSC0 - master, start 
        ldi     temp,(1<<PARUN1)
        sts     PCTL1,temp
        ldi     temp,(1<<PARUN2)
        sts     PCTL2,temp
        ldi     temp,(1<<PRUN0)
        sts     PCTL0,temp


        StartDDS
	ret

;----------------------------------------------------------------------------

SinTab:

.include "Sin128.asm"		;sine wave table

;----------------------------------------------------------------------------
