//----------------------------------------------------------------------------

//3-phase generator SG-633
//  

//----------------------------------------------------------------------------

#include "Main.h"
#include "Control.h"
#include "Menu.h"

//----------------------------- : -----------------------------------

#define BAUD1 1250000 //   

#define C_SET_F	0x01 //SET FREQUENCY command
#define C_SET_P	0x02 //SET PHASE command
#define C_SET_D	0x03 //SET DELTA PHASE command
#define C_SET_A	0x04 //SET AMPLITUDE command

#define CHAN_A	0x10 //channel A mask
#define CHAN_B	0x20 //channel B mask
#define CHAN_C	0x40 //channel C mask

#define UBRRV1 (int)((F_CLK * 1000000.0)/(16.0 * BAUD1) - 0.5) //
#define TX_CMD (1 << RXEN1) | (1 << TXEN1) | (1 << UCSZ12) | (1 << TXB81)
#define TX_DAT (1 << RXEN1) | (1 << TXEN1) | (1 << UCSZ12)

#define DDS_DIV  512            //    DDS   
#define DDS_RES  33554432LL     // DDS (2^25)
#define CALF_RES 10000000LL     //   (0.1 PPM)
#define PHS_RES (DDS_RES / 1)   //   DDS (2^25)

#define KF ((long long)(DDS_RES * DDS_DIV / 2048))
#define DF ((long long)(F_CLK * 1000000000LL / 2048)) 

#define SIGCAL  0xBABE // EEPROM  
#define CALL_RES 10000 //   (0.01%)

//  :

#define MAX_TAB 1100 //    , 0.01 

const int __flash Leveltab[MAX_TAB + 1] = {
#include "Leveltab.h"
};

//----------------------------- : ----------------------------------

static int  Phase[OUTS]; //  , 3600 x0.1

static int  Cal_F;       // , 999 (99.9 ppm)
static int  Cal_A[OUTS]; // , 999 (9.99%)

const int __flash Log[1001] = {
#include "Log.tab"
};

__no_init __eeprom int  ESigCal;      // EEPROM  
__no_init __eeprom int  ECal_F;       //   EEPROM
__no_init __eeprom int  ECal_A[OUTS]; //   EEPROM

//--------------------------  : ------------------------------

void CoByte(char b);          //  

//-----------------------  : --------------------------

void Control_Init(void)
{
  // UART1,  9 :
	UCSR1C = (1 << UCSZ11) | (1 << UCSZ10);
  // 9  :
	UCSR1B = TX_DAT;
  //  :
  UBRR1L = LO(UBRRV1);                 
  UBRR1H = HI(UBRRV1);
  // :
  Control_ReadCal();
  // :
  for(char i = 0; i < OUTS; i++)
  {
    Phase[i] = 0;
  }
  //co-processor power-up delay 100 mS:
  Delay_ms(100);
}

//---------------------    EEPROM: -------------------------

void Control_ReadCal(void)
{
  if(ESigCal != SIGCAL) //  ,
  {
    // :
    Cal_F = NOM_CALIB;
    for(char i = 0; i < OUTS; i++)
    {
      Cal_A[i] = NOM_CALIB;
    }
    Control_SaveCal(); //   EEPROM
  }
  else //  ,
  {
    Cal_F = ECal_F; //  
    for(char i = 0; i < OUTS; i++) //  
    {
      Cal_A[i] = ECal_A[i];
    }
  }
}

//-------------------    EEPROM: ------------------------

void Control_SaveCal(void)
{
  if(ECal_F != Cal_F) ECal_F = Cal_F; //  
  for(char i = 0; i < OUTS; i++)      //  
  {
    if(ECal_A[i] != Cal_A[i]) ECal_A[i] = Cal_A[i];
    __watchdog_reset();
  }
  if(ESigCal != SIGCAL) ESigCal = SIGCAL; // 
}

//------------------   : ---------------------

int Control_GetK(char s)
{
  switch(s)
  {
  case CAL_F: return(Cal_F);
  case CAL_A: return(Cal_A[OUT_A]);
  case CAL_B: return(Cal_A[OUT_B]);
  case CAL_C: return(Cal_A[OUT_C]);
  }
  return(0);
}

//-----------------   : -------------------

void Control_SetK(int k, char s)
{
  switch(s)
  {
  case CAL_F: Cal_F = k; break;
  case CAL_A: Cal_A[OUT_A] = k; break;
  case CAL_B: Cal_A[OUT_B] = k; break;
  case CAL_C: Cal_A[OUT_C] = k; break;
  }
}

//-----------------------   : ---------------------

void CoCommand(char cmd, long d)
{
	UCSR1B = TX_CMD;  //  (TXB81 = 1)
  CoByte(cmd);
	UCSR1B = TX_DAT;  //  (TXB81 = 0)
  CoByte(BYTE1(d));
  CoByte(BYTE2(d));
  CoByte(BYTE3(d));
  CoByte(BYTE4(d));
}

//---------------------   : -------------------------

void CoByte(char b)
{
  char CoInt = Pin_CINT; //  Pin_CINT
  char TmOut = 222;      //  (~100 )
  UDR1 = b;              //   
  while(Pin_CINT == CoInt && TmOut--); //  Pin_CINT
}

//---------------------------  : -----------------------------

//ch - : OUT_A, OUT_B, OUT_C, OUTS
//f - : 0..50000000 x0.001 Hz

void Control_SetF(char ch, long f)
{
  char cmd; long code;
  if(ch == OUT_A) cmd = C_SET_F + CHAN_A;
  if(ch == OUT_B) cmd = C_SET_F + CHAN_B;
  if(ch == OUT_C) cmd = C_SET_F + CHAN_C;
  if(ch == OUTS)  cmd = C_SET_F + CHAN_A + CHAN_B + CHAN_C;
  code = (long long)f * (KF + Cal_F * KF / CALF_RES) / DF;
  CoCommand(cmd, code);
}

//-------------------------   : ---------------------------

//ch - : OUT_A, OUT_B, OUT_C, OUTS
//p -  : 3600 x0.1

void Control_SetP(char ch, int p)
{
  char cmd; int dp = p; long code;
  if(ch == OUT_A) { cmd = C_SET_D + CHAN_A; dp = p - Phase[ch]; Phase[ch] = p; }
  if(ch == OUT_B) { cmd = C_SET_D + CHAN_B; dp = p - Phase[ch]; Phase[ch] = p; }
  if(ch == OUT_C) { cmd = C_SET_D + CHAN_C; dp = p - Phase[ch]; Phase[ch] = p; }
  if(ch == OUTS)  { cmd = C_SET_P + CHAN_A + CHAN_B + CHAN_C;
                    Phase[OUT_A] = p; Phase[OUT_B] = p; Phase[OUT_C] = p; } 
  code = (long long)dp * PHS_RES / MAX_P;
  CoCommand(cmd, code);
}

//----------------------------  : -----------------------------

//ch - : OUT_A, OUT_B, OUT_C, OUTS
//l - : 0..1000 x0.01 V

void Control_SetA(char ch, int l)
{
  long Cal = Cal_A[ch];
  int level = (long)l * (MAX_A + Cal * MAX_A / CALL_RES) / MAX_A;
  if(level < 0) level = 0;
  if(level > MAX_TAB) level = MAX_TAB;
  int code = Leveltab[level];
  
  if(ch == OUT_A || ch == OUTS) CoCommand(C_SET_A + CHAN_A, code);
  if(ch == OUT_B || ch == OUTS) CoCommand(C_SET_A + CHAN_B, code);
  if(ch == OUT_C || ch == OUTS) CoCommand(C_SET_A + CHAN_C, code);
}

//----------------------------------------------------------------------------
