//----------------------------------------------------------------------------

//3-phase generator SG-633
// : header file

//----------------------------------------------------------------------------

#ifndef MainH
#define MainH

//----------------------------------------------------------------------------

#include <iom324p.h>
#include <intrinsics.h>
#include <stdbool.h>
#include <stdlib.h>

//----------------------------------------------------------------------------

#define VER "1.0"     //  firmware

#define	F_CLK 8.0000  //clock frequency, MHz
#define T_SYS 1000    //system timer, uS

//----------------------------------------------------------------------------

// :

//I - 
//O - 
//B -  

//L -   
//H -   
//X -    
//A -  

//-------------------------------  A: ------------------------------------

#define KEY_E    (1 << PA0) //IL -  
#define ENC_F1   (1 << PA1) //IL -   F1
#define ENC_F2   (1 << PA2) //IL -   F2
#define RES      (1 << PA3) //OL - LCD reset
#define NC_PA4   (1 << PA4) //IL -  
#define NC_PA5   (1 << PA5) //IL -  
#define NC_PA6   (1 << PA6) //IL -  
#define NC_PA7   (1 << PA7) //IL -  

//:
#define I_DDRA  (RES)
//  (in)    (out):
#define I_PORTA (KEY_E | ENC_F1 | ENC_F2 | NC_PA4 | NC_PA5 | NC_PA6 | NC_PA7)
//     :
#define Pin_KEY_E      (PINA & KEY_E)
#define Pin_ENC_F1     (PINA & ENC_F1)
#define Pin_ENC_F2     (PINA & ENC_F2)
#define Port_RES_0     (PORTA &= ~RES)
#define Port_RES_1     (PORTA |= RES)

//-------------------------------  B: ------------------------------------

#define KEY_C    (1 << PB0) //IL -  "CHAN"
#define KEY_U    (1 << PB1) //IL -  "UP"
#define KEY_D    (1 << PB2) //IL -  "DOWN"
#define KEY_S    (1 << PB3) //IL -  "STEP"
#define NC_PB4   (1 << PB4) //IL -  
#define MOSI     (1 << PB5) //OH -  MOSI  SPI
#define MISO     (1 << PB6) //IH -  MISO  SPI
#define SCK      (1 << PB7) //OH -  SCK  SPI

//:
#define I_DDRB  (MOSI | SCK)
//  (in)    (out):
#define I_PORTB (KEY_C | KEY_U | KEY_D | KEY_S | NC_PB4 | MISO)
//     :
#define Pin_KEY_C      (PINB & KEY_C)
#define Pin_KEY_U      (PINB & KEY_U)
#define Pin_KEY_D      (PINB & KEY_D)
#define Pin_KEY_S      (PINB & KEY_S)

//-------------------------------  C: ------------------------------------

#define SCL      (1 << PC0) //OL -  SCL  I2C
#define SDA      (1 << PC1) //BL -  SDA  I2C
#define NC_PC2   (1 << PC2) //IL -  
#define NC_PC3   (1 << PC3) //IL -  
#define NC_PC4   (1 << PC4) //IL -  
#define NC_PC5   (1 << PC5) //IL -  
#define NC_PC6   (1 << PC6) //IL -  
#define NC_PC7   (1 << PC7) //IL -  

//:
#define I_DDRC  (SCL)
//  (in)    (out):
#define I_PORTC (SCL | SDA | NC_PC2 | NC_PC3 | NC_PC4 | NC_PC5 | NC_PC6 | NC_PC7)

//-------------------------------  D: ------------------------------------

#define PC_RXD0  (1 << PD0) //IL -  RXD0 (PC)
#define PC_TXD0  (1 << PD1) //OL -  TXD0 (PC)
#define PC_RXD1  (1 << PD2) //IL -  RXD1 (DDS)
#define PC_TXD1  (1 << PD3) //OL -  TXD1 (DDS)
#define CINT     (1 << PD4) //IL - DDS CINT
#define SND      (1 << PD5) //OL -  
#define NC_PD6   (1 << PD6) //IL -  
#define NC_PD7   (1 << PD7) //IL -  

//:
#define I_DDRD  (PC_TXD0 | PC_TXD1 | SND)
//  (in)    (out):
#define I_PORTD (PC_RXD0 | PC_TXD0 | PC_RXD1 | PC_TXD1 | CINT | SND | NC_PD6 | NC_PD7)
//     :
#define Pin_CINT       (PIND & CINT)
#define Port_SND       (PORTD & SND)
#define Port_SND_0     (PORTD &= ~SND)
#define Port_SND_1     (PORTD |= SND)

//--------------------------- : ------------------------------

#define HI(x) ((x >> 8) & 0xFF)
#define LO(x) (x & 0xFF)

#define BYTE1(x) (x & 0xFF)
#define BYTE2(x) ((unsigned int)(x >> 8) & 0xFF)
#define BYTE3(x) ((unsigned long)(x >> 16) & 0xFF)
#define BYTE4(x) ((unsigned long)(x >> 24) & 0xFF)

#define WORD(b1,b0) ((unsigned int)(b1 << 8) | b0)
#define DWORD(b3,b2,b1,b0) (((unsigned long)WORD(b3,b2) << 16) | WORD(b1,b0))

#define Delay_us(x) __delay_cycles((int)(x * F_CLK + 0.5))
#define ms2sys(x) ((int)(1E3 * x / T_SYS))

//--------------------------  : ------------------------------

void Delay_ms(int d);

//----------------------------------------------------------------------------

#endif
